;; Nome Completo:
;; RA:

;; Este eh um modelo para implementacao dos jogadores em Lisp. Nele
;; voce encontra a definicao das 3 funcoes requeridas pelo
;; gerenciador de campeonato (start, play e last_play). Observe
;; que este arquivo contem apenas definicoes validas, mas nao
;; implementa um jogador que realize jogadas validas.
;;
;; IMPORTANTE: 
;;
;; 1 - As chamadas feitas ao jogador devem ser respondidas em, no
;; maximo, 1 segundo. Extrapolar este tempo resultara na
;; desclassificacao do jogador.
;;
;; 2 - Nao use acentuacao, mesmo dentro dos comentarios.
;;
;; 3 - Seu jogador nao deve imprimir nada na tela. A funcao
;; print-field (e seus auxiliares) deve ser usada apenas para "debug"
;; (via linha de comando no clisp).
;;
;; RECOMENDACOES:
;;
;; 1 - Represente a bola por uma variavel global ball. Exemplo:
;;
;;   (setf ball '(4 . 5))
;;
;; 2 - Represente os robos brancos e pretos de atraves de listas, uma
;; para cada cor de robo. Neste caso use variaveis globais com nomes
;; whites e blacks. Exemplos:
;;
;;     (setf whites '((2 . 1) (3 . 4) (1 . 3) (1 . 5) ... (2 . 3)))
;;     (setf blacks '((5 . 3) (5 . 2) (5 . 6) (5 . 7) ... (6 . 4)))
;;
;;   Observacao: cada posicao do campo de batalha deve ser
;;   representada atraves de um par com ponto na forma:
;;   (linha . coluna)
;;
;; 3 - Represente o numero de linhas e colunas usando uma variavel
;; global columns. Exemplo:
;;
;;   (setf columns 7)

;; START: Esta funcao eh chamada apenas uma vez por partida pelo
;; gerenciador de campeonatos. Eh a primeira chamada feita ao jogador
;; no inicio de cada partida e eh atraves desta chamada que o jogador
;; toma conhecimento sobre o campo de batalha, que iniciara a
;; partida. O primeiro parametro refere-se, aos numeros de linhas/colunas
;; do campo de batalha. O segundo parametro refere-se a posicao inicial
;; da bola, representada por um par com pontos no formato (linha . coluna).
;; Os dois ultimos parametros sao, respectivamente, as listas com as
;; posicoes iniciais dos robos brancos (robos do jogador) e dos robos
;; pretos (robos do adversario), onde cada posicao eh representada
;; no formato (linha . coluna). A funcao start deve sempre retornar T.
(defun start (Colunas Bola Brancos Pretos)
  T
  )

;; LAST-PLAY: Esta funcao eh utilizada pelo gerenciador para indicar a
;; ultima jogada realizada pelo jogador oponente (robos pretos),
;; atraves de dois parametros, Old e New, onde Old e New sao posicoes
;; do campo de batalha e cada posicao eh representada no formato
;; (linha . coluna). A funcao last-play deve sempre retornar T.
(defun last-play (Old New) 
  T
  )

;; PLAY: Esta funcao eh utilizada pelo gerenciador para solicitar a
;; proxima jogada (dos robos brancos). O jogador deve retornar uma lista
;; contendo duas posicoes (Old e New), onde cada posicao eh
;; representada no formato (linha . coluna).
(defun play ()
  ; Exemplo de retorno:
  ; (list '(1 . 1) '(2 . 2))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;; Interface com o usuario ;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; Verifica se um elemento faz parte de uma lista

(defun my-member (elemento lista)
  (cond ((null lista) nil)
	((equal elemento (car lista)) lista)
	(t (my-member elemento (cdr lista)))
	)
  )

;; Imprime o campo de batalha

(defun print-field ()
  (dotimes (r (+ columns 2) t)
    (let ((linha (1+ (- columns r))))
      (progn 
	(headline linha)
	(dotimes (c columns)
	  (let* ((coluna (1+ c))
		 (pos (cons linha coluna)))
	    (print-pos pos)
	    )
	  )
	(terpri)
	)
      )
    )
  )

;; Imprime o cabecalho de uma linha do campo de batalha
  
(defun headline (row)
  (unless (> row 9)
      (write 0)
      )
  (write row)
  (write-char #\space)
  )

;; Imprime uma posicao do campo de batalha

(defun print-pos (pos)
  (if (equal pos ball)
      (cond ((my-member pos whites) (write-char #\W))
	    ((my-member pos blacks) (write-char #\B))
	    (t (write-char #\o))
	    )
    (cond ((my-member pos whites) (write-char #\w))
	  ((my-member pos blacks) (write-char #\b))
	  (t (write-char #\.))
	  )
    )
  )
